#ident	"@(#)util.c	1.2"
/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/resource.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdarg.h>
#include <stdlib.h>
#include <errno.h>
#include <time.h>
#include <signal.h>
#include <netdb.h>
#include <ctype.h>
#include <slipcmd.h>

#define LOGPERM	(S_IRUSR|S_IWUSR|S_IRGRP)
#define BUFSZ 1024

int logfd = -1;
static char buf[BUFSZ];

static void _cmnerr(const char *fmt, va_list);
int strcmp(const char *, const char *);

void
error(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	_cmnerr(fmt, ap);
	va_end(ap);
}

void
fatal(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	_cmnerr(fmt, ap);
	va_end(ap);
	exit(1);
}

static void
_cmnerr(const char *fmt, va_list ap)
{
	int err;

	err = errno;
	vfprintf(stderr, fmt, ap);
	if (err != 0)
		fprintf(stderr, ": %s", strerror(err));
	fputc('\n', stderr);
}

int
initlog(const char *logfile)
{
	struct stat sbuf;

	logfd = open(logfile, O_WRONLY|O_APPEND|O_CREAT, LOGPERM);
	if (logfd < 0)
		return(-1);
	fcntl(logfd, F_SETFD, FD_CLOEXEC);
	if (fstat(logfd, &sbuf) < 0) {
		close(logfd);
		logfd = -1;
		return(-1);
	}
	if ((sbuf.st_uid != 0) || (sbuf.st_gid != 0))
		fchown(logfd, 0, 0);
	return(logfd);
}

void
log(const char *fmt, ...)
{
	time_t t;
	char *tm;
	va_list ap;
	struct stat sbuf;

	time(&t);
	tm = ctime(&t);
	tm[24] = '\t';	/* change newlint to tab */
	strncpy(buf, tm, 25);
	va_start(ap, fmt);
	vsprintf(buf+25, fmt, ap);
	va_end(ap);
	strcat(buf, "\n");
	if (fstat(logfd, &sbuf) == 0) {
		if (sbuf.st_size > 100000)
			ftruncate(logfd, 0);
	}
	write(logfd, buf, strlen(buf));
}

void
daemonize(const char *cmd)
{
	int i;
	pid_t pid;
	struct rlimit rl;
	char filename[256];

	umask(0);
	sprintf(filename, "/tmp/%s.log", cmd);
	if (initlog(filename) < 0)
		fatal("%s: cannot create log file", cmd);
	if (getrlimit(RLIMIT_NOFILE, &rl) < 0)
		fatal("%s: cannot get file limit", cmd);
	if ((pid = fork()) < 0)
		fatal("%s: cannot fork", cmd);
	else if (pid != 0)	/* parent */
		exit(0);
	setsid();
	sigset(SIGHUP, SIG_IGN);
	if ((pid = fork()) < 0)
		fatal("%s: cannot fork", cmd);
	else if (pid != 0)	/* parent */
		exit(0);
	if (rl.rlim_max == RLIM_INFINITY)
		rl.rlim_max = 1024;
	for (i = 0; i < rl.rlim_max; i++)
		if (i != logfd)
			close(i);
	errno = 0;
	if (chdir("/") < 0) {
		log("cannot chdir to /: %s", strerror(errno));
		exit(1);
	}
	if ((i = open("/dev/null", O_RDWR)) != 0) {
		if (i < 0)
			log("error opening /dev/null: %s", strerror(errno));
		else
			log("open /dev/null: expected fd 0, got fd %d", i);
		exit(1);
	}
	if ((i = dup(0)) != 1) {
		if (i < 0)
			log("dup failed: %s", strerror(errno));
		else
			log("dup(0) expected fd 1, got fd %d", i);
		exit(1);
	}
	if ((i = dup(0)) != 2) {
		if (i < 0)
			log("dup failed: %s", strerror(errno));
		else
			log("dup(0) expected fd 2, got fd %d", i);
		exit(1);
	}
}

struct conninfo *
getcinfo(char *host)
{
	struct conninfo *cip;
	int found, idx, start;
	FILE *fp;
	char name[MAXHOSTNAMELEN];

	cip = calloc(1, sizeof(struct conninfo));
	if (cip == NULL)
		return(NULL);
	fp = fopen(SLIPSYS, "r");
	if (fp == NULL)
		goto err;
	found = 0;
	while (fgets(buf, sizeof(buf), fp) != NULL) {
		buf[strlen(buf)-1] = '\0';	/* get rid of newline */
		if ((strlen(buf) == 0) || (buf[0] == '#'))
			continue;
		if (sscanf(buf, "%*s %s", name) != 1)
			continue;
		if (strcmp(name, host) == 0) {
			found = 1;
			break;
		}
	}
	if (found) {
		idx = 0;
		while (isspace(buf[idx]))
			idx++;
		start = idx;
		while (isdigit(buf[idx]) || (buf[idx] == '.'))
			idx++;
		buf[idx++] = '\0';
		cip->laddr = strdup(&buf[start]);
		if (cip->laddr == NULL)
			goto err;
		cip->fhost = strdup(name);
		if (cip->fhost == NULL)
			goto err;
		while (isspace(buf[idx]))
			idx++;
		idx += strlen(name);
		while (isspace(buf[idx]))
			idx++;
		if (buf[idx] == '\0')
			goto err;
		start = idx;
		while (isdigit(buf[idx]) || (buf[idx] == '.'))
			idx++;
		if (buf[idx] == '\0')
			goto err;
		buf[idx++] = '\0';
		cip->faddr = strdup(&buf[start]);
		if (cip->faddr == NULL)
			goto err;
		while (isspace(buf[idx]))
			idx++;
		if (buf[idx] == '\0')
			goto err;
		cip->chatscript = strdup(&buf[idx]);
		if (cip->chatscript == NULL)
			goto err;
		fclose(fp);
		return(cip);
	} else {
		errno = EINVAL;
	}
err:
	if (fp != NULL)
		fclose(fp);
	if (cip != NULL)
		freecinfo(cip);
	return(NULL);
}

void
freecinfo(struct conninfo *cip)
{
	if (cip->laddr != NULL)
		free(cip->laddr);
	if (cip->fhost != NULL)
		free(cip->fhost);
	if (cip->faddr != NULL)
		free(cip->faddr);
	if (cip->chatscript != NULL)
		free(cip->chatscript);
	free(cip);
}

int
strcmp(register const char *s1, register const char *s2)
{
	if(s1 == s2)
		return(0);
	while(*s1 == *s2++)
		if(*s1++ == '\0')
			return(0);
	return(*s1 - *--s2);
}
